#!/bin/bash

# Copyright (C) 2018  Tetras Libre <Contact@Tetras-Libre.fr>
# Author: Beniamine, David <David.Beniamine@Tetras-Libre.fr>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

mainlog="/var/log/`basename $0`_`date +%Y%m%d-%H%M`"

getips(){
    awk '/^lease/{IP=$2} /hardware ethernet/{gsub(";","",$3);if($3 == "'$1'"){print IP}}' \
        < /var/lib/dhcp/dhcpd.leases | sort -u
}

runcmd(){
    host=$1
    logfile=/tmp/$$-$host.log
    shift
    echo "Runnig $@ on $host" | tee $logfile
    ssh -o StrictHostKeychecking=no root@$host $@ | tee $logfile
}

log(){
    echo $@ | tee -a $mainlog
}

# Switch to lametros repository
DIR=`dirname $0`
$DIR/update_repository.sh

conf_dir="/etc/registry"
conf_file="$conf_dir/hosts"
broadcast=$(ip a | awk '/inet.*eth1/ {print $4}' | cut -d / -f 1)
wait_time=120
wait_step=5

if [ ! -z "$2" ]
then
    # $1 == Apt, $2 == hostname or $@ == command
    if [ "$1" == "Apt" ]
    then
        arg="apt"
        host="$2"
    else
        arg="$@"
    fi
elif [ ! -z "$1" ]
then
    # $1 == Apt, hostname or $@ == command
    if [ "$1" == "Apt" ]
    then
        arg="apt"
    elif [ ! -z "$(grep $1 /etc/registry/hosts)" ]
    then
        host="$1"
    else
        arg="$@"
    fi
fi

cmd="cd /opt/primtux-eole/; export HTTPS_PROXY='http://proxy:3128'; git pull; ./upgrade-session.sh $arg"
echo $cmd
declare -A HOSTS
declare -A IPTOMACS
declare -A DOWNS

if [ -z "$host" ]
then
    catcmd="cat $conf_file"
else
    catcmd="grep $host $conf_file"
fi

while read date mac name endline
do
    # Boot machine
    if [ ! -z "$date" ]
    then
        HOSTS[$mac]=$name
        log "Trying to boot $name ($mac)"
        for i in seq 1 3
        do
            wakeonlan -i $broadcast "$mac"
        done
        sleep 5
        MACS="$MACS $mac"
    fi
done < <($catcmd)

# Wait for boot
log "Wakeonlan sent, waiting"
while [ $wait_time -gt 0 ]
do
    if [ ! -z "$host" ]
    then
        leave=false
        # try to skip useless wait for single upgrade
        for ip in $(getips $MACS)
        do
            ping -c 3 $ip
            if [ $? -eq 0 ]
            then
                leave=true
            fi
        done
        if $leave
        then
            break
        fi
    fi
    log -ne "Waiting for boot, remaining $wait_time seconds\r"
    sleep $wait_step
    wait_time=$(($wait_time - $wait_step))
done
log -e "\nWaiting finished"

# Running command
for mac in $MACS
do
    for ip in $(getips $mac)
    do
        IPTOMACS[$ip]=$mac
        runcmd $ip $cmd &
        PIDS="$PIDS $!"
    done
done
# wait all pids
for pid in $PIDS
do
    wait $pid
done

# Retrieving logs
for f in /tmp/$$-*
do
    ip=$(echo $f | cut -d '-' -f 2- | sed 's/\.log$//')
    mac=${IPTOMACS[$ip]}
    name=${HOSTS[$mac]}
    log "===== Begin host $name ($mac - $ip) ===="
    cat $f >> $mainlog
    log "===== End host $name ===="
    if [ $(wc -l $f | awk '{print $1}') -lt 0 ]
    then
        DOWNS[$mac]=$name
    fi
    rm $f
done

# Listing down hosts
if [ ${#DOWNS[@]} -eq 0 ]
then
    log "All ${#HOSTS[@]} hosts answered"
else
    log "On a total of ${#HOSTS[@]} hosts, ${#DOWNS[@]} are still down:"
    for mac in ${!DOWNS[@]}
    do
        log -e "\t* ${DOWNS[$mac]} ($mac)"
    done
fi
log "Output written to $mainlog"
